{% extends 'appearance/base.html' %}

{% load i18n %}
{% load static %}

{% load autoadmin_tags %}
{% load common_tags %}
{% load motd_tags %}

{% block base_title %}{% trans 'Login' %}{% endblock %}

{% block project_name %}{% endblock %}

{% block content_plain %}
    {% autoadmin_properties %}
    {% if autoadmin_properties.account %}
        <div class="row">
            <div class="col-xs-10 col-xs-offset-1 col-sm-10 col-sm-offset-1 col-md-8 col-md-offset-2 col-lg-6 col-lg-offset-3">
                <div class="panel panel-primary">
                    <div class="panel-heading">
                        <h3 class="panel-title">{% trans "First time login" %}</h3>
                    </div>
                    <div class="panel-body">
                        <p>{% trans 'You have just finished installing <strong>Mayan EDMS</strong>, congratulations!' %}</p>
                        <p>{% trans 'Login using the following credentials:' %}</p>
                        <p>{% blocktrans with autoadmin_properties.account as account %}Username: <strong>{{ account }}</strong>{% endblocktrans %}</p>
                        <p>{% blocktrans with autoadmin_properties.account.email as email %}Email: <strong>{{ email }}</strong>{% endblocktrans %}</p>
                        <p>{% blocktrans with autoadmin_properties.password as password %}Password: <strong>{{ password }}</strong>{% endblocktrans %}</p>
                        <p>{% trans 'Be sure to change the password to increase security and to disable this message.' %}</p>
                    </div>
                </div>
            </div>
        </div>
    {% endif %}

    {% motd %}

    <div class="row">
        <div class="col-xs-10 col-xs-offset-1 col-sm-8 col-sm-offset-2 col-md-6 col-md-offset-3 col-lg-4 col-lg-offset-4">
            <div class="panel panel-primary">
                <div class="panel-heading">
                    <h3 class="panel-title">&nbsp;</h3>
                </div>
                <div class="panel-body">
                    <h2>{% trans 'Sign in' %}</h2>
                    <hr>
                    <div class="content login">
                        <form action="." method="post" class="form login">{% csrf_token %}
                            <div class="group wat-cf">
                                {% include 'appearance/generic_form_instance.html' %}
                                <input type="hidden" name="next" value="{{ next|escape }}" />

                            <div class="form-group">
                                <button class="btn btn-primary" name="{% if form.prefix %}{{ form.prefix }}-submit{% else %}submit{% endif %}" type="submit"><i class="fa fa-sign-in"></i> {% trans 'Sign in' %}</button>
                            </div>

                        </form>
                    </div>
                </div>
            </div>
        </div>
    </div>
{% endblock %}
